@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2013 uim Project https://github.com/uim/uim
 
   All rights reserved.
 
@@ -48,7 +48,8 @@ enum UCustomType {
   UCustom_Pathname,
   UCustom_Choice,
   UCustom_OrderedList,
-  UCustom_Key
+  UCustom_Key,
+  UCustom_Table
 };
 
 enum UCustomPathnameType {
@@ -74,6 +75,7 @@ union uim_custom_value {
   struct uim_custom_choice *as_choice;
   struct uim_custom_choice **as_olist;
   struct uim_custom_key **as_key;
+  char ***as_table;
 };
 
 struct uim_custom_pathname {
@@ -111,6 +113,10 @@ union uim_custom_range {
   struct {
     struct uim_custom_choice **valid_items;  /* contains all possible items */
   } as_olist;
+
+  struct {
+    struct uim_custom_choice **valid_items;
+  } as_table_header;
 };
 
 struct uim_custom {
@@ -136,6 +142,7 @@ uim_bool uim_custom_enable(void);
 /* load & save */
 uim_bool uim_custom_load(void);
 uim_bool uim_custom_save(void);
+uim_bool uim_custom_save_custom(const char *custom_sym);
 uim_bool uim_custom_broadcast(void);
 uim_bool uim_custom_broadcast_reload_request(void);
 
