@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2013 uim Project https://github.com/uim/uim
 
   All rights reserved.
 
@@ -37,13 +37,12 @@
 
 #include <stdlib.h>
 
+#include "version.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#define UIM_VERSION_MAJOR      (1)
-#define UIM_VERSION_MINOR      (5)
-#define UIM_VERSION_PATCHLEVEL (3)
 #define UIM_VERSION_REQUIRE(major, minor, patchlevel)			     \
   ((major) < UIM_VERSION_MAJOR						     \
    || ((major) == UIM_VERSION_MAJOR && (minor) < UIM_VERSION_MINOR)	     \
@@ -84,7 +83,108 @@ enum UKey {
   UKey_7 = 55,
   UKey_8 = 56,
   UKey_9 = 57,
-  UKey_Yen = 165,
+
+  /* non-ASCII Latin-1 characters */
+  UKey_nobreakspace   = 0x00a0,
+  UKey_exclamdown     = 0x00a1,
+  UKey_cent           = 0x00a2,
+  UKey_sterling       = 0x00a3,
+  UKey_currency       = 0x00a4,
+  UKey_Yen            = 0x00a5,	/* == 165 */
+  UKey_brokenbar      = 0x00a6,
+  UKey_section        = 0x00a7,
+  UKey_diaeresis      = 0x00a8,
+  UKey_copyright      = 0x00a9,
+  UKey_ordfeminine    = 0x00aa,
+  UKey_guillemotleft  = 0x00ab,
+  UKey_notsign        = 0x00ac,
+  UKey_hyphen         = 0x00ad,
+  UKey_registered     = 0x00ae,
+  UKey_macron         = 0x00af,
+  UKey_degree         = 0x00b0,
+  UKey_plusminus      = 0x00b1,
+  UKey_twosuperior    = 0x00b2,
+  UKey_threesuperior  = 0x00b3,
+  UKey_acute          = 0x00b4,
+  UKey_mu             = 0x00b5,
+  UKey_paragraph      = 0x00b6,
+  UKey_periodcentered = 0x00b7,
+  UKey_cedilla        = 0x00b8,
+  UKey_onesuperior    = 0x00b9,
+  UKey_masculine      = 0x00ba,
+  UKey_guillemotright = 0x00bb,
+  UKey_onequarter     = 0x00bc,
+  UKey_onehalf        = 0x00bd,
+  UKey_threequarters  = 0x00be,
+  UKey_questiondown   = 0x00bf,
+  UKey_Agrave         = 0x00c0,
+  UKey_Aacute         = 0x00c1,
+  UKey_Acircumflex    = 0x00c2,
+  UKey_Atilde         = 0x00c3,
+  UKey_Adiaeresis     = 0x00c4,
+  UKey_Aring          = 0x00c5,
+  UKey_AE             = 0x00c6,
+  UKey_Ccedilla       = 0x00c7,
+  UKey_Egrave         = 0x00c8,
+  UKey_Eacute         = 0x00c9,
+  UKey_Ecircumflex    = 0x00ca,
+  UKey_Ediaeresis     = 0x00cb,
+  UKey_Igrave         = 0x00cc,
+  UKey_Iacute         = 0x00cd,
+  UKey_Icircumflex    = 0x00ce,
+  UKey_Idiaeresis     = 0x00cf,
+  UKey_ETH            = 0x00d0,
+  UKey_Eth            = 0x00d0,
+  UKey_Ntilde         = 0x00d1,
+  UKey_Ograve         = 0x00d2,
+  UKey_Oacute         = 0x00d3,
+  UKey_Ocircumflex    = 0x00d4,
+  UKey_Otilde         = 0x00d5,
+  UKey_Odiaeresis     = 0x00d6,
+  UKey_multiply       = 0x00d7,
+  UKey_Oslash         = 0x00d8,
+  UKey_Ooblique       = 0x00d8,
+  UKey_Ugrave         = 0x00d9,
+  UKey_Uacute         = 0x00da,
+  UKey_Ucircumflex    = 0x00db,
+  UKey_Udiaeresis     = 0x00dc,
+  UKey_Yacute         = 0x00dd,
+  UKey_THORN          = 0x00de,
+  UKey_Thorn          = 0x00de,
+  UKey_ssharp         = 0x00df,
+  UKey_agrave         = 0x00e0,
+  UKey_aacute         = 0x00e1,
+  UKey_acircumflex    = 0x00e2,
+  UKey_atilde         = 0x00e3,
+  UKey_adiaeresis     = 0x00e4,
+  UKey_aring          = 0x00e5,
+  UKey_ae             = 0x00e6,
+  UKey_ccedilla       = 0x00e7,
+  UKey_egrave         = 0x00e8,
+  UKey_eacute         = 0x00e9,
+  UKey_ecircumflex    = 0x00ea,
+  UKey_ediaeresis     = 0x00eb,
+  UKey_igrave         = 0x00ec,
+  UKey_iacute         = 0x00ed,
+  UKey_icircumflex    = 0x00ee,
+  UKey_idiaeresis     = 0x00ef,
+  UKey_eth            = 0x00f0,
+  UKey_ntilde         = 0x00f1,
+  UKey_ograve         = 0x00f2,
+  UKey_oacute         = 0x00f3,
+  UKey_ocircumflex    = 0x00f4,
+  UKey_otilde         = 0x00f5,
+  UKey_odiaeresis     = 0x00f6,
+  UKey_division       = 0x00f7,
+  UKey_oslash         = 0x00f8,
+  UKey_ooblique       = 0x00f8,
+  UKey_ugrave         = 0x00f9,
+  UKey_uacute         = 0x00fa,
+  UKey_ucircumflex    = 0x00fb,
+  UKey_udiaeresis     = 0x00fc,
+  UKey_yacute         = 0x00fd,
+  UKey_thorn          = 0x00fe,
+  UKey_ydiaeresis     = 0x00ff,
 
   UKey_Escape = 256,
   UKey_Tab,
@@ -688,6 +788,31 @@ void uim_set_candidate_selector_cb(uim_context uc,
                                                          int direction),
                                    void (*deactivate_cb)(void *ptr));
 
+/**
+ * Set callback function to support delay showing candidate-selection.
+ *
+ * @param uc input context
+ * @param delay_activate_cb called when candidate window should be activated with delay.
+ *
+ * @see uim_create_context
+ */
+void uim_set_delay_candidate_selector_cb(uim_context uc,
+                                         void (*delay_activate_cb)(void *ptr,
+                                                                   int delay));
+
+/**
+ * Notify that the candidate selector is being activated after delay.
+ *
+ * The input context must update number of candidates,
+ * display limit and selected index.
+ *
+ * @param uc input context
+ * @param nr [out] total number of candidates
+ * @param display_limit [out] number of candidates to show on one page
+ * @param selected_index [out] index of selected candidate
+ */
+void uim_delay_activating(uim_context uc, int *nr, int *display_limit, int *selected_index);
+
 /**
  * Get candidate data.
  *
@@ -827,7 +952,7 @@ uim_set_mode_list_update_cb(uim_context uc,
  * completely, adding separated surrounding text deletion interface is better
  * solution at now.  -- YamaKen 2006-10-07
  *
- * http://uim.googlecode.com/svn/branches/composer/scm/event.scm
+ * https://github.com/uim/uim/blob/composer/scm/event.scm
  *
  * (define-event 'commit
  *   upward-event-rec-spec
@@ -926,6 +1051,8 @@ void *uim_malloc(size_t size);
 void *uim_realloc(void *p, size_t size);
 void *uim_calloc(size_t nmemb, size_t size);
 char *uim_strdup(const char *s);
+int uim_asprintf(char **, const char *, ...);
+
 
 #ifdef __cplusplus
 }
