@@ -1,5 +1,5 @@
 ;; 
-;;  Copyright (c) 2005-2007 uim Project http://uim.freedesktop.org/
+;;  Copyright (c) 2005-2013 uim Project https://github.com/uim/uim
 ;;
 ;;  All rights reserved.
 ;;
@@ -50,7 +50,7 @@
 (defun uim-merge-candidate ()
 
   (let ((maxwidth (string-width (nth 1 (car uim-candidate-line-list))))
-	offset mark-base mark-cursor)
+	offset mark-base mark-cursor mark-limit)
 
     ;; format page label
     (setq uim-candidate-page-label
@@ -66,13 +66,21 @@
 
     ;; mark current-point
     (setq mark-base (point-marker))
-    ;;(uim-debug (format "before: %s" (marker-position mark-base)))
+
+    ;; save font-lock face
+    (when uim-xemacs
+      (uim-overwrite-font-face uim-candidate-original-start
+			       uim-candidate-original-end))
 
     ;; save original string
     (setq uim-candidate-original-str
 	  (buffer-substring uim-candidate-original-start
 			    uim-candidate-original-end))
 
+    (save-excursion
+      (goto-char uim-candidate-original-end)
+      (setq mark-limit (point-marker)))
+
     ;; remove tabs/spaces
     (uim-tab-pad-space uim-candidate-original-start
 		       uim-candidate-original-end)
@@ -83,8 +91,6 @@
       (setq base-ofs (- (marker-position mark-base) uim-candidate-start))
       (setq uim-candidate-start (+ uim-candidate-start base-ofs)))
 
-    ;;(uim-debug (format "after: %s" (marker-position mark-base)))
-
     (set-marker mark-base nil)
 
     (goto-char uim-candidate-start)
@@ -94,8 +100,6 @@
       (setq offset 
 	    (uim-string-width (buffer-substring (point) uim-candidate-start))))
 
-    ;;(uim-debug (format "offset: %s" offset))
-
     ;; if offset + maxwidth >= window-width then reduce offset
     (if (>= (+ offset maxwidth 2) (window-width))
 	(setq offset (- (- (window-width) 1) 
@@ -153,78 +157,87 @@
 
 		  (setq vhead (point))
 
-		  ;; make new line
-		  (save-excursion
+		  (if (<= (save-excursion
+			    (end-of-line)
+			    (point))
+			  (marker-position mark-limit))
+		  
+		      ;; make new line
+		      (save-excursion
 
-		    (end-of-line)
+			(end-of-line)
 		    
-		    ;; |ABCDEFGH\ + |___aaa__|
-		    ;; |IJKLMN  |
-		    ;;
-		    ;; get region from vhead to physical end
-		    ;;
-		    ;; linetmp = ABCDEFGH
-		    ;; 
-		    (setq linetmp 
-			  (truncate-string-to-width (buffer-substring vhead (point))
-						    (window-width)))
+			;; |ABCDEFGH\ + |___aaa__|
+			;; |IJKLMN  |
+			;;
+			;; get region from vhead to physical end
+			;;
+			;; linetmp = ABCDEFGH
+			;; 
+			(setq linetmp 
+			      (truncate-string-to-width (buffer-substring vhead (point))
+							(window-width)))
 
-		    ;; make padding by truncating linetmp to offset size
-		    ;;  (add space as padding if shortage)
-		    ;;   witdth of padding is equal to offset
-		    ;; 
-		    ;; padding = ABC
-		    ;;
-		    (setq padding
-			  (truncate-string-to-width linetmp offset nil 32))
+			;; make padding by truncating linetmp to offset size
+			;;  (add space as padding if shortage)
+			;;   witdth of padding is equal to offset
+			;; 
+			;; padding = ABC
+			;;
+			(setq padding
+			      (truncate-string-to-width linetmp offset nil 32))
 		      
-		    ;;  overflow = GH
-		    ;;
-		    (let ((candwidth (+ maxwidth 2)))
+			;;  overflow = GH
+			;;
+			(let ((candwidth (+ maxwidth 2)))
 
-		      (if (>= (uim-string-width linetmp) (+ offset candwidth))
-			  (setq overflow 
-				(truncate-string-to-width linetmp 
-							  (uim-string-width linetmp)
-							  (+ offset candwidth) 32))
-			(setq overflow "")))
+			  (if (>= (uim-string-width linetmp) (+ offset candwidth))
+			      (setq overflow 
+				    (truncate-string-to-width linetmp 
+							      (uim-string-width linetmp)
+							      (+ offset candwidth) 32))
+			    (setq overflow "")))
 
-		    ;; index string selected appendix
-		    (save-excursion
-		      (goto-char vhead)
-		      (delete-region vhead (+ vhead (length linetmp)))
+			;; index string selected appendix
+			(save-excursion
+			  (goto-char vhead)
+			  (delete-region vhead (+ vhead (length linetmp)))
 			
-		      (when uim-xemacs
-			(insert " ")
-			(remove-text-properties (- (point) 1) (point)
-						'(face nil))
-			(goto-char (- (point) 1)))
+			  (when uim-xemacs
+			    (insert " ")
+			    (remove-text-properties (- (point) 1) (point)
+						    '(face nil))
+			    (goto-char (- (point) 1)))
 
-		      (insert (concat padding 
-				      candstr
-				      overflow))
+			  (insert (concat padding 
+					  candstr
+					  overflow))
 
-		      (when uim-xemacs
-			(delete-char 1))
+			  (when uim-xemacs
+			    (delete-char 1))
 		      
-		      )
+			  )
 		
-		    (uim-set-candidate-face candidx candsel 
-					    (+ vhead (length padding))
-					    (length candstr))
+			(uim-set-candidate-face candidx candsel 
+						(+ vhead (length padding))
+						(length candstr))
 
 
-		    ;; update merge-end
-		    (save-excursion
-		      (end-of-line)
-		      (setq uim-candidate-end (point)))
+			;; update merge-end
+			(save-excursion
+			  (end-of-line) 
+			  (setq uim-candidate-end 
+			      (if (> (marker-position mark-limit) (point))
+				  (marker-position mark-limit)
+				(point))))
 	      
-		    (setq mergecount (- mergecount 1))
+			(setq mergecount (- mergecount 1))
+			)
 		    )
 
 		  (if uim-show-candidate-upward
-		  (uim-vertical-motion 1))
-		    
+		      (uim-vertical-motion 1))
+
 		  )
 	      ;; append 
 	      (let (vhead)
@@ -243,7 +256,9 @@
 
 		(save-excursion
 		  (end-of-line)
-		  (setq uim-candidate-end (point))))))
+		  (setq uim-candidate-end (point)))
+
+		)))
 	    
 	  (if uim-candidate-line-list
 	      (setq uim-candidate-line-list
@@ -252,22 +267,20 @@
 
 	  )
 	
+	(force-mode-line-update)
+
+	;; update cursor position
+	(goto-char (marker-position mark-cursor))
+	(setq uim-candidate-cursor (point))
+
 	;; move to end of preedit
 	(if uim-show-candidate-upward
 	    (save-excursion
 	      (end-of-line)
 	      (setq uim-candidate-end (point))))
 
-	(force-mode-line-update)
+	(set-marker mark-cursor nil)
 
-	;; update cursor position
-	(when mark-cursor
-	  
-	  (goto-char (marker-position mark-cursor))
-	  (setq uim-candidate-cursor (point))
-
-	  (set-marker mark-cursor nil)
-	  )
 	))
     ))
 
@@ -276,12 +289,21 @@
 (defun uim-echo-candidate (cand)
 
   ;; display candidate in minibuffer 
-
-  (let ((cands "")
+  (let* ((cands "")
 	(selstart 0)
 	(selend 0)
 	(page-current (format "%d" (caar cand)))
 	(page-total (format "%d" (cdar cand)))
+	 (cand-height 0)
+	 (displayed nil)
+	 (echo-max-height 1)
+	 (page-space (- (string-width page-total) 
+			(string-width page-current)))
+	 (page-label (concat (if (> page-space 0) 
+				 (make-string page-space 32))
+			     page-current  "/" page-total " "))
+	 cands-tmp
+	 (uim-candidate-in-echo-region t)
 	)
 
     (setq cand (cdr cand))
@@ -308,56 +330,85 @@
 	) cand)
 
     
-    (if (and uim-emacs
+    (setq cands-tmp (concat page-label cands))
+    
+    (if (and uim-allow-resize-echo-region
+	     uim-emacs
 	     (>= emacs-major-version 21))
 	;; Emcas-21 or Emacs-22
-	(let ((page-space (- (string-width page-total) 
-			     (string-width page-current)))
-	      message-log-max)
-	  (setq cands 
-		(concat (if (> page-space 0) (make-string page-space 32))
-			page-current  "/" page-total " " cands))
-	  (message cands)
-	  )
-      ;; Emacs-20 or XEmacs
-      (let* ((page-space (- (string-width page-total) 
-			    (string-width page-current)))
-	     (page-label (concat (if (> page-space 0) 
-				     (make-string page-space 32))
-				  page-current  "/" page-total " "))
-	     (page-width (string-width page-label))
-	     (cands-width (string-width cands))
-	     (echoreg-width (- (- (window-width) 1) page-width)))
+	(let (message-log-max)
 
-	(cond ((>= echoreg-width cands-width)
-	       (setq cands
-		     (concat page-label cands)))
+	  (save-excursion
+	    (set-buffer (get-buffer-create uim-el-candidates-buffer-name))
+	    (erase-buffer)
+	    (insert cands-tmp)
+	    (setq cand-height (+ (uim-vertical-distance 1 (buffer-end 1)) 1))
+	    (let ((current-minibuf (window-buffer (minibuffer-window))))
+	      (set-window-buffer (minibuffer-window) 
+				 uim-el-candidates-buffer-name)
+	      (set-buffer (window-buffer (minibuffer-window)))
+	      ;; estimate height of candidates in mini buffer
+	      (setq cand-height (+ (uim-vertical-distance 1 (buffer-end 1)) 1))
+	      (set-window-buffer (minibuffer-window) current-minibuf))
+
+	    (message cands-tmp)
+	    (if (= (setq echo-max-height (window-height (minibuffer-window)))
+		   cand-height)
+		(setq displayed t)))))
+    
+    (if (not displayed)
+	;; Emacs-20 or XEmacs or uim-allow-resize-echo-region is nil 
+	;; or too small frame
+	(let* ((trimmed "")
+	     (page-width (string-width page-label))
+	       (page-cands-width (string-width cands-tmp))
+	     (cands-width (string-width cands))
+	       (candreg-width (- (- (* (window-width) echo-max-height) 1)
+				 page-width)))
+
+	  (cond ((>= candreg-width cands-width)
+		 (setq trimmed cands-tmp))
+
+		((<= (window-width) page-width)
+		 ;; this part might not be reached
+		 (setq trimmed (truncate-string-to-width page-label
+							 (- (window-width) 1))))
+
+		((<= (window-width) (+ page-width 6))
+		 (setq trimmed (concat page-label
+				       (make-string 
+					(- (window-width) page-width 1)
+
+					?.))))
 
 	      ((= selstart 0)
 	       ;; | 10/134 [1.xxxxx] 2.yyyyy 3.zzzzz ...|
 	       (setq cands 
 		     (truncate-string-to-width cands
-					       (if (> echoreg-width 3)
-						   (- echoreg-width 3)
+						 (if (> candreg-width 3)
+						     (- candreg-width 3)
 						 0)))
-	       (setq cands
+		 (setq trimmed
 		     (concat page-label
 			     cands
-			     (make-string (- echoreg-width
+			       (make-string (- candreg-width
 					     (string-width cands))
 					  ?.)))
 
 	       )
-	      ((> (string-width (substring cands 0 selstart))
-		  (+ (- cands-width echoreg-width) 3))
+
+		((<= (string-width (substring cands selstart))
+		     (- candreg-width 3)) ;; at least 1 dot
 	       ;; | 10/134 ...yyy [3.zzzzz] 
 	       (setq cands
 		     (truncate-string-to-width cands
 					       cands-width 
-					       (+ (- cands-width echoreg-width) 3)))
-	       (setq cands
+						 (+ (- cands-width 
+						       candreg-width) 3)))
+
+		 (setq trimmed
 		     (concat page-label
-			     (make-string (- echoreg-width 
+			       (make-string (- candreg-width 
 					     (string-width cands))
 					  ?.)
 			     cands)))
@@ -366,21 +417,17 @@
 	       (setq cands 
 		     (concat "..."
 			     (truncate-string-to-width (substring cands selstart)
-						       (if (> echoreg-width 6)
-							   (- echoreg-width 6)
-							 0))
-			     ))
-	       (setq cands
+							 (if (> candreg-width 6)
+							     (- candreg-width 6)
+							   0))))
+		 (setq trimmed
 		     (concat page-label
 			     cands
-			     (make-string (- echoreg-width 
+			       (make-string (- candreg-width 
 					     (string-width cands))
-					  ?.))))))
-
+					    ?.)))))
       (let (message-log-max)
-	(message cands))
-      )
-    
+	    (message trimmed))))
     )
   )
 
@@ -486,7 +533,6 @@
 ;;
 (defun uim-show-candidate (candidate)
 
-
   ;; separate appendix (for prime...)
   (setq candidate
 	(cons (car candidate)
@@ -529,16 +575,16 @@
   (setq uim-candidate-original-start nil)
   (setq uim-candidate-original-end nil)
 
-  (let ((display-inline uim-candidate-display-inline))
+  (let ((display-inline uim-candidate-display-inline)
+	(upward-end-line nil))
 
     (if (>= (max (+ uim-max-candlabel 6) 
 		 (+ (string-width uim-candidate-page-label) 2))
 	    (window-width))
-	(progn
-	  (setq display-inline nil)
-	  (uim-debug "disable inline: window is too thin")
-	  ))
+	(setq display-inline nil))
 
+    (if (/= (window-hscroll) 0)
+	(setq display-inline nil))
 
     (if display-inline
 	(uim-make-candlist (cdr candidate)))
@@ -548,7 +594,9 @@
 	(save-excursion
 	  (goto-char uim-candidate-start)
 
-	  (let ((winofs (uim-get-window-offset))
+	  (let ((winofs (save-excursion
+			  (goto-char uim-candidate-cursor)
+			  (uim-get-window-offset)))
 		(candlines (+ 1 (length uim-candidate-line-list)))
 		ol-down dhead dtail uhead utail)
 	    
@@ -561,15 +609,17 @@
 					    (+ uim-candidate-vofs 
 					       candlines))
 					   (end-of-line)
-					   (setq dtail(point))))))
+					   (setq dtail (point))))))
+		     ;; 
+		     (save-excursion
+		       (goto-char uim-candidate-cursor)
 		     (>= (- (- (window-height) 1) (+ winofs 1)) 
-			 candlines))
+			   (+ candlines 1))))
 
 		(progn
 		  (setq uim-show-candidate-upward nil)
-		  (setq uim-candidate-original-start dhead)
-		  (setq uim-candidate-original-end dtail)
-		  )
+		  (setq uim-candidate-original-start dhead)		  
+		  (setq uim-candidate-original-end dtail))
 
 	      (if (and (>= winofs candlines)
 		       (not (uim-check-overlay
@@ -582,43 +632,40 @@
 			       (setq utail (point)))))
 		       )
 		  (progn
+		    (save-excursion
+		      (setq upward-end-line (uim-get-window-offset)))
 		    (setq uim-show-candidate-upward t)
 		    (setq uim-candidate-original-start uhead)
-		    (setq uim-candidate-original-end utail)
-		    )
+		    (setq uim-candidate-original-end utail))
 		
-		;; scroll anyway
-		(if (not ol-down)
-		    (let* ((fspace (- (- (- (window-height) 1) winofs) 1))
-			   (vshift (- candlines fspace)))
+		;; Scroll buffer to show all candidates
+		(if (and (not ol-down)
+			 (>= (- (window-height) 1) (+ candlines 1)))
+		    (progn
 		      (setq uim-show-candidate-upward nil)
+		      (if (not uim-window-force-scrolled-original)
+			  (setq uim-window-force-scrolled-original 
+				(window-start)))
 		      (setq uim-window-force-scrolled t)
-
 		      (setq uim-candidate-original-start dhead)
 		      (setq uim-candidate-original-end dtail)
 
-		      (if (> vshift winofs)
-			  (recenter 0)
 			(save-excursion 
-			  ;;(uim-debug (format "vshift %s" vshift))
-			  (uim-vertical-motion (- (- (window-height)
-						     candlines 2)))
-			  (recenter 0))))
+			  (goto-char uim-candidate-cursor)
+			  (recenter (- (- (- (window-height) 1) candlines) 1))))
 		  ;; disable inline display
 		  (setq display-inline nil)
 		  
-		  ;;(uim-debug (format "ol-down %s" ol-down))
-		  
 		  ))))))
 
-    (if (not display-inline)
-	(uim-debug "disable inline"))
-
-
     (if display-inline
 	;; inline candidate display mode
 	(let ((inhibit-read-only t))
-	  (uim-merge-candidate))
+	  (uim-merge-candidate)
+	  (if uim-show-candidate-upward 
+	      (save-excursion
+		(recenter upward-end-line)))
+	  )
       ;; display in echo region
       (uim-echo-candidate candidate)
       )
