@@ -1,5 +1,5 @@
 ;; 
-;;  Copyright (c) 2006-2008 uim Project http://code.google.com/p/uim/
+;;  Copyright (c) 2006-2013 uim Project https://github.com/uim/uim
 ;;
 ;;  All rights reserved.
 ;;
@@ -85,7 +85,7 @@
 
 
 (defun uim-helper-process-sentinel (proc stat)
-  (message "uim.el: %s" stat)
+  (message "uim-helper.el: %s" stat)
   (setq uim-el-helper-agent-process nil)
   )
 
@@ -106,7 +106,7 @@
     (if (not proc)
 	(error "uim.el: Couldn't invoke uim-el-helper-agent."))
 
-    (process-kill-without-query proc)
+    (set-process-query-on-exit-flag proc nil)
 
     ;; wait "OK"
     (let ((patience uim-startup-timeout) (ok nil))
