@@ -1,6 +1,6 @@
 ;;; custom.scm: Customization support
 ;;;
-;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2013 uim Project https://github.com/uim/uim
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -65,7 +65,8 @@
     (pathname     . custom-pathname?)
     (choice       . custom-valid-choice?)
     (ordered-list . custom-ordered-list?)
-    (key          . custom-key?)))
+    (key          . custom-key?)
+    (table        . custom-table?)))
 
 (define anything?
   (lambda (x)
@@ -119,6 +120,18 @@
 			   (custom-exist? key 'key))))
 		key-repls))))
 
+(define custom-table?
+  (lambda arg
+    (let ((syms (car arg))
+          (choice-rec-alist (cdr arg)))
+      (and (list? syms)
+           (every (lambda (row)
+                    (and (list? row)
+                         (every (lambda (cell)
+                                  (string? cell))
+                                row)))
+                  syms)))))
+
 (define custom-pathname-type
   (lambda (custom-sym)
     (car (custom-type-attrs custom-sym))))
@@ -647,7 +660,7 @@
     (let* ((type (custom-type sym))
 	   (attrs (custom-type-attrs sym)))
       (case type
-	((choice ordered-list)
+	((choice ordered-list table)
 	 (map custom-choice-rec-sym attrs))
 	((integer string)
 	 attrs)
@@ -677,6 +690,18 @@
 			      lst)))
       (string-append "'(" (string-join canonicalized " ") ")"))))
 
+; (custom-list-as-table  '(("a" "b" "c") ("d" "e")))
+;   -> "'((\"a\" \"b\" \"c\") (\"d\" \"e\"))"
+(define custom-list-as-table
+  (lambda (tbl)
+    (string-append "'("
+      (string-join
+        (map (lambda (lst)
+          (string-append "(" 
+            (string-join
+              (map (lambda (elem)
+                (string-escape elem)) lst) " ") ")")) tbl) " ") ")")))
+
 ;; API
 (define custom-value-as-literal
   (lambda (sym)
@@ -694,6 +719,8 @@
        ((or (eq? type 'ordered-list)
 	    (eq? type 'key))
 	(custom-list-as-literal val))
+       ((eq? type 'table)
+	(custom-list-as-table val))
        ((or (eq? val #f)
 	    (eq? type 'boolean))
 	(if (eq? val #f)
