@@ -1,7 +1,7 @@
 ;;; mana.scm: mana for uim.
 ;;; charset: EUC-JP
 ;;;
-;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2013 uim Project https://github.com/uim/uim
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -33,8 +33,6 @@
 (require "util.scm")
 (require "ustr.scm")
 (require "japanese.scm")
-(require "japanese-kana.scm")
-(require "japanese-azik.scm")
 (require-custom "generic-key-custom.scm")
 (require-custom "mana-custom.scm")
 (require-custom "mana-key-custom.scm")
@@ -143,8 +141,10 @@
 	   (left-str (ustr-former-seq raw-str)))
      (append left-str
 	     (if residual-kana
-		 (list pending)
-		 '())
+               (if (list? (car residual-kana))
+                 (reverse (string-to-list pending))
+                 (list pending))
+               '())
 	     right-str))))
 
 (define mana-get-raw-candidate
@@ -173,7 +173,9 @@
 	   (if (member (car unconv) preconv)
 	       (let ((start (list-seq-contained? preconv unconv))
 		     (len (length unconv)))
-		 (if start
+		 (if (and
+                       start
+                       (= (length raw-str) (length preconv))) ;; sanity check
 		     (mana-make-raw-string
 		      (reverse (sublist-rel raw-str start len))
 		      (if (or
@@ -277,6 +279,8 @@
 (define mana-input-rule-roma 0)
 (define mana-input-rule-kana 1)
 (define mana-input-rule-azik 2)
+(define mana-input-rule-act 3)
+(define mana-input-rule-kzik 4)
 
 (define mana-candidate-type-katakana -2)
 (define mana-candidate-type-hiragana -3)
@@ -465,6 +469,7 @@
                       mana-input-rule-kana))
                  (lambda (mc)
                    (mana-prepare-input-rule-activation mc)
+                   (require "japanese-kana.scm")
                    (mana-context-set-input-rule! mc mana-input-rule-kana)
                    (mana-context-change-kana-mode!
 		    mc (mana-context-kana-mode mc))
@@ -487,10 +492,47 @@
                       mana-input-rule-azik))
                  (lambda (mc)
                    (mana-prepare-input-rule-activation mc)
+                   (require "japanese-azik.scm")
                    (rk-context-set-rule! (mana-context-rkc mc)
                                          ja-azik-rule)
                    (mana-context-set-input-rule! mc mana-input-rule-azik)))
 
+(register-action 'action_mana_kzik
+                 ;;              (indication-alist-indicator 'action_mana_kzik
+                 ;;                                          mana-kana-input-method-indication-alist)
+                 (lambda (mc)
+                   '(ja_kzik
+                      ""
+                      "KZIK"
+                      "KZIKĥ޻ϥ⡼"))
+                 (lambda (mc)
+                   (= (mana-context-input-rule mc)
+                      mana-input-rule-kzik))
+                 (lambda (mc)
+                   (mana-prepare-input-rule-activation mc)
+                   (require "japanese-kzik.scm")
+                   (rk-context-set-rule! (mana-context-rkc mc)
+                                         ja-kzik-rule)
+                   (mana-context-set-input-rule! mc mana-input-rule-kzik)))
+
+(register-action 'action_mana_act
+                 (lambda (mc)
+                   '(ja_act
+                      ""
+                      "ACT"
+                      "ACTĥ޻ϥ⡼"))
+                 (lambda (mc)
+                   (= (mana-context-input-rule mc)
+                      mana-input-rule-act))
+                 (lambda (mc)
+                   (mana-prepare-input-rule-activation mc)
+                   (require "japanese-act.scm")
+                   (rk-context-set-rule! (mana-context-rkc mc)
+                                         ja-act-rule)
+                   (mana-context-set-input-rule! mc mana-input-rule-act)))
+
+
+
 ;; Update widget definitions based on action configurations. The
 ;; procedure is needed for on-the-fly reconfiguration involving the
 ;; custom API
@@ -542,15 +584,6 @@
       (mana-context-set-preconv-ustr! mc (ustr-new '()))
       (mana-context-set-raw-ustr! mc (ustr-new '()))
       (mana-context-set-segments! mc (ustr-new '()))
-
-      ;; 2004-08-26 Takuro Ashie <ashie@homa.ne.jp>
-      ;;   * I think load-kana-table should be marked as depracated.
-      ;;     Because it is a little violent (it overwrites ja-rk-rule table).
-      ;;     We should prepare a custom entry like "uim-default-input-rule"
-      ;;     instead of using-kana-table.
-      (if using-kana-table?
-          (mana-context-set-input-rule! mc mana-input-rule-kana)
-          (mana-context-set-input-rule! mc mana-input-rule-roma))
       mc)))
 
 (define mana-commit-raw
@@ -603,8 +636,10 @@
 	      (string-append-map-ustr-former extract-kana preconv-str)
 	      (if convert-pending-into-kana?
 		  (if residual-kana
-		      (extract-kana residual-kana)
-		      pending)
+                    (if (list? (car residual-kana))
+                      (string-append-map extract-kana residual-kana)
+		      (extract-kana residual-kana))
+                    pending)
 		  pending)
 	      (string-append-map-ustr-latter extract-kana preconv-str))))
 	   kana)
@@ -612,8 +647,10 @@
 	   (string-append-map-ustr-former extract-kana preconv-str)
 	   (if convert-pending-into-kana?
 	       (if residual-kana
-		   (extract-kana residual-kana)
-                   "")
+                 (if (list? (car residual-kana))
+                   (string-append-map extract-kana residual-kana)
+		   (extract-kana residual-kana))
+                 "")
 	       pending)
 	   (string-append-map-ustr-latter extract-kana preconv-str))))))
 
@@ -875,8 +912,9 @@
 		    (res (rk-push-key! rkc key-str)))
 	       (if res
 		   (begin
-		     (ustr-insert-elem! (mana-context-preconv-ustr mc)
-					res)
+                     (if (list? (car res))
+                       (ustr-insert-seq! (mana-context-preconv-ustr mc) res)
+                       (ustr-insert-elem! (mana-context-preconv-ustr mc) res))
 		     (ustr-insert-elem! (mana-context-raw-ustr mc)
 					key-str))
 		   (if (null? (rk-context-seq rkc))
@@ -994,6 +1032,12 @@
 
 (define mana-proc-input-state-with-preedit
   (lambda (mc key key-state)
+    (define (check-auto-conv str)
+      (and
+	str
+	mana-auto-start-henkan?
+	(string-find japanese-auto-start-henkan-keyword-list str)
+	(mana-begin-conv mc)))
     (let ((preconv-str (mana-context-preconv-ustr mc))
           (raw-str (mana-context-raw-ustr mc))
           (rkc (mana-context-rkc mc))
@@ -1166,22 +1210,6 @@
         #f)
 
        (else
-	;; handle "n1" sequence as "1"
-	(if (and (not (mana-context-alnum mc))
-		 (not (ichar-alphabetic? key))
-		 (not (string-find
-		       (rk-expect rkc)
-		       (charcode->string
-			(if (= rule mana-input-rule-kana)
-			    key
-			    (ichar-downcase key))))))
-	    (let ((pend (rk-pending rkc))
-		  (residual-kana (rk-push-key-last! rkc)))
-	      (if residual-kana
-		  (begin
-		    (ustr-insert-elem! preconv-str residual-kana)
-		    (ustr-insert-elem! raw-str pend)))))
-
 	(if (mana-context-alnum mc)
 	    (let ((key-str (charcode->string key))
 		  (pend (rk-pending rkc))
@@ -1189,15 +1217,22 @@
 	      (rk-flush rkc) ;; OK to reset rkc here.
 	      (if residual-kana
 		  (begin
-		    (ustr-insert-elem! preconv-str residual-kana)
-		    (ustr-insert-elem! raw-str pend)))
+                    (if (list? (car residual-kana))
+                      (begin
+                        (ustr-insert-seq! preconv-str residual-kana)
+                        (ustr-insert-elem! raw-str (reverse
+                                                     (string-to-list pend))))
+                      (begin
+                        (ustr-insert-elem! preconv-str residual-kana)
+                        (ustr-insert-elem! raw-str pend)))))
 	      (ustr-insert-elem! preconv-str
 				 (if (= (mana-context-alnum-type mc)
 					mana-type-halfwidth-alnum)
 				     (list key-str key-str key-str)
 				     (list (ja-wide key-str) (ja-wide key-str)
 					   (ja-wide key-str))))
-	      (ustr-insert-elem! raw-str key-str))
+	      (ustr-insert-elem! raw-str key-str)
+	      (check-auto-conv key-str))
 	    (let* ((key-str (charcode->string 
 			     (if (= rule mana-input-rule-kana)
 				 key
@@ -1213,14 +1248,25 @@
 			(ustr-insert-elem! preconv-str res))
 		    (if (and next-pend
 			     (not (string=? next-pend "")))
-			(ustr-insert-elem! raw-str pend)
+			(ustr-insert-seq! raw-str
+                                          (reverse (string-to-list pend)))
 			(if (list? (car res))
 			    (begin
-			      (ustr-insert-elem! raw-str pend)
+                              (if (member pend
+                                          (map car
+                                               ja-consonant-syllable-table))
+                                ;; treat consonant having more than one
+                                ;; charactear as one raw-str in this case
+                                (ustr-insert-elem! raw-str pend)
+                                (ustr-insert-seq! raw-str (reverse
+                                                            (string-to-list
+                                                              pend))))
+                              ;; assume key-str as a vowel
 			      (ustr-insert-elem! raw-str key-str))
 			    (ustr-insert-elem!
 			     raw-str
-			     (string-append pend key-str)))))))))))))
+			     (string-append pend key-str))))))
+	      (check-auto-conv (if res (car res) #f)))))))))
 
 (define mana-context-confirm-kana!
   (lambda (mc)
@@ -1231,7 +1277,9 @@
                (residual-kana (rk-peek-terminal-match rkc)))
           (if residual-kana
               (begin
-                (ustr-insert-elem! preconv-str residual-kana)
+                (if (list? (car residual-kana))
+                  (ustr-insert-seq! preconv-str residual-kana)
+                  (ustr-insert-elem! preconv-str residual-kana))
                 (rk-flush rkc)))))))
 
 (define mana-proc-input-state
