@@ -1,6 +1,6 @@
 ;;; prime-custom.scm: Customization variables for prime.scm
 ;;;
-;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2013 uim Project https://github.com/uim/uim
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -89,12 +89,51 @@
   (N_ "Alternative space character key")
   (N_ "long description will be here."))
 
-(define-custom 'prime-use-unixdomain? #t
+(define-custom 'prime-server-setting? 'unixdomain
   '(prime advanced)
-  '(boolean)
-  (N_ "Use UNIX domain socket to communicate with PRIME")
+  (list 'choice
+        (list 'unixdomain
+              (N_ "Unix domain")
+              (N_ "Use UNIX domain socket to communicate with PRIME."))
+        (list 'tcpserver
+              (N_ "TCP server")
+              (N_ "Use tcp server to communicate with PRIME"))
+        (list 'pipe
+              (N_ "Pipe")
+              (N_ "Use pipe. spawn new prime process to communicate with PRIME")))
+  (N_ "Prime connection setting")
   (N_ "long description will be here."))
 
+(define-custom 'prime-command-path "prime"
+  '(prime advanced)
+  '(pathname regular-file)
+  (N_ "Prime command path")
+  (N_ "long description will be here."))
+
+(define-custom 'prime-tcpserver-name "localhost"
+  '(prime advanced)
+  '(string ".*")
+  (N_ "PRIME server address")
+  (N_ "long description will be here."))
+
+(custom-add-hook 'prime-tcpserver-name
+		 'custom-activity-hooks
+		 (lambda ()
+                   (eq? prime-server-setting?
+                        'tcpserver)))
+
+(define-custom 'prime-tcpserver-port 1180
+  '(prime advanced)
+  '(integer 0 65535)
+  (N_ "PRIME server port")
+  (N_ "long description will be here."))
+
+(custom-add-hook 'prime-tcpserver-port
+		 'custom-activity-hooks
+		 (lambda ()
+                   (eq? prime-server-setting?
+                        'tcpserver)))
+
 ;(define-custom 'prime-use-candidate-window? #t
 ;  '(prime candwin)
 ;  '(boolean)
@@ -141,6 +180,19 @@
   (N_ "Show candidate forms")
   (N_ "long description will be here."))
 
+(custom-add-hook 'prime-custom-display-usage?
+  'custom-activity-hooks
+  (lambda ()
+    enable-annotation?))
+(custom-add-hook 'prime-custom-display-comment?
+  'custom-activity-hooks
+  (lambda ()
+    enable-annotation?))
+(custom-add-hook 'prime-custom-display-form?
+  'custom-activity-hooks
+  (lambda ()
+    enable-annotation?))
+
 ;; ------------------------------------------------------------
 
 (define-custom 'prime-custom-number-selection? #f
@@ -214,7 +266,7 @@
 ;; Users don't care this option.
 ;; (2005-03-03) <Hiroyuki Komatsu>
 ;(define-custom 'default-widget_prime_input_mode 'action_prime_mode_latin
-;  '(prime-advanced toolbar)
+;  '(prime-advanced toolbar-widget)
 ;  (cons 'choice
 ;	(map indication-alist-entry-extract-choice
 ;	     prime-input-mode-indication-alist))
@@ -226,7 +278,7 @@
 ;; (2005-03-03) <Hiroyuki Komatsu>
 ;(define-custom 'prime-input-mode-actions
 ;               (map car prime-input-mode-indication-alist)
-;   '(prime-advanced toolbar)
+;   '(prime-advanced toolbar-widget)
 ;  (cons 'ordered-list
 ;	(map indication-alist-entry-extract-choice
 ;	     prime-input-mode-indication-alist))
