@@ -52,16 +52,16 @@
     ;((("c" "1"))(""))
     ((("r" "1"))(""))
     ((("l" "1"))(""))
-    ((("a" "1"))(""))
+    ((("a" "1"))("" ""))
     ((("o" "1"))("õ"))
     ((("e" "1"))(""))
     ((("u" "1"))(""))
     ((("i" "1"))(""))
-    ((("d" "1"))(""))
+    ((("d" "1"))("" ""))
     ((("h" "1"))(""))
     ((("t" "1"))(""))
     ((("n" "1"))(""))
-    ((("s" "1"))(""))
+    ((("s" "1"))("" ""))
     (((";" "1"))(""))
     ((("q" "1"))(""))
     ((("j" "1"))(""))
@@ -142,7 +142,7 @@
     ((("t" "3"))(""))
     ((("n" "3"))(""))
     ((("s" "3"))(""))
-    (((";" "3"))(""))
+    (((";" "3"))("" ""))
     ((("q" "3"))(""))
     ((("j" "3"))(""))
     ((("k" "3"))(""))
@@ -483,7 +483,7 @@
     ;((("8" "."))(""))
     ;((("9" "."))(""))
     ;((("0" "."))(""))
-    ((("," "."))(""))
+    ((("," "."))("" ""))
     ((("." "."))(""))
     ((("p" "."))(""))
     ((("y" "."))(""))
@@ -492,7 +492,7 @@
     ((("c" "."))(""))
     ((("r" "."))(""))
     ((("l" "."))(""))
-    ((("a" "."))(""))
+    ((("a" "."))("" ""))
     ((("o" "."))(""))
     ((("e" "."))(""))
     ((("u" "."))(""))
@@ -503,12 +503,12 @@
     ((("n" "."))(""))
     ((("s" "."))("Ÿ"))
     (((";" "."))(""))
-    ((("q" "."))(""))
+    ((("q" "."))("" ""))
     ((("j" "."))(""))
     ((("k" "."))(""))
     ((("x" "."))(""))
     ((("b" "."))("ͷ"))
-    ((("m" "."))(""))
+    ((("m" "."))("" ""))
     ((("w" "."))(""))
     ((("v" "."))(""))
     ((("z" "."))(""))
@@ -593,7 +593,7 @@
     ((("v" "y"))("Ψ"))
     ((("z" "y"))(""))
     ((("/" "y"))(""))
-    ((("1" "f"))(""))
+    ((("1" "f"))("" ""))
     ((("2" "f"))(""))
     ((("3" "f"))(""))
     ((("4" "f"))(""))
@@ -603,10 +603,10 @@
     ((("8" "f"))(""))
     ((("9" "f"))(""))
     ((("0" "f"))("ɸ"))
-    ((("," "f"))(""))
+    ((("," "f"))("" ""))
     ((("." "f"))(""))
     ((("p" "f"))(""))
-    ((("y" "f"))(""))
+    ((("y" "f"))("" ""))
     ((("f" "f"))(""))
     ((("g" "f"))(""))
     ((("c" "f"))(""))
@@ -615,15 +615,15 @@
     ((("a" "f"))(""))
     ((("o" "f"))(""))
     ((("e" "f"))(""))
-    ((("u" "f"))(""))
+    ((("u" "f"))("" ""))
     ((("i" "f"))("6"))
     ((("d" "f"))(""))
     ((("h" "f"))(""))
     ((("t" "f"))(""))
-    ((("n" "f"))(""))
+    ((("n" "f"))("" ""))
     ((("s" "f"))("ͽ"))
     (((";" "f"))("2"))
-    ((("q" "f"))(""))
+    ((("q" "f"))("" ""))
     ((("j" "f"))(""))
     ((("k" "f"))("0"))
     ((("x" "f"))(""))
@@ -633,7 +633,7 @@
     ((("v" "f"))(""))
     ((("z" "f"))(""))
     ((("/" "f"))(""))
-    ((("1" "g"))(""))
+    ((("1" "g"))("" ""))
     ((("2" "g"))(""))
     ((("3" "g"))(""))
     ((("4" "g"))(""))
@@ -651,29 +651,29 @@
     ((("g" "g"))(""))
     ((("c" "g"))(""))
     ((("r" "g"))(""))
-    ((("l" "g"))(""))
+    ((("l" "g"))("" ""))
     ((("a" "g"))(""))
     ((("o" "g"))(""))
     ((("e" "g"))(""))
-    ((("u" "g"))(""))
-    ((("i" "g"))(""))
+    ((("u" "g"))("" ""))
+    ((("i" "g"))("" ""))
     ((("d" "g"))(""))
-    ((("h" "g"))(""))
+    ((("h" "g"))("" ""))
     ((("t" "g"))("ʹ"))
     ((("n" "g"))(""))
     ((("s" "g"))(""))
-    (((";" "g"))(""))
+    (((";" "g"))("" ""))
     ((("q" "g"))(""))
-    ((("j" "g"))(""))
-    ((("k" "g"))(""))
+    ((("j" "g"))("" ""))
+    ((("k" "g"))("" ""))
     ((("x" "g"))(""))
     ((("b" "g"))(""))
     ((("m" "g"))(""))
     ((("w" "g"))(""))
     ((("v" "g"))(""))
     ((("z" "g"))("ž"))
-    ((("/" "g"))(""))
-    ((("1" "c"))(""))
+    ((("/" "g"))("" ""))
+    ((("1" "c"))("" ""))
     ;((("2" "c"))(""))
     ;((("3" "c"))(""))
     ;((("4" "c"))(""))
@@ -689,22 +689,22 @@
     ((("y" "c"))(""))
     ((("f" "c"))(""))
     ((("g" "c"))(""))
-    ((("c" "c"))(""))
+    ((("c" "c"))("" ""))
     ((("r" "c"))(""))
-    ((("l" "c"))(""))
+    ((("l" "c"))("" ""))
     ((("a" "c"))(""))
     ((("o" "c"))(""))
     ((("e" "c"))("λ"))
     ((("u" "c"))(""))
     ((("i" "c"))("ϻ"))
     ((("d" "c"))(""))
-    ((("h" "c"))(""))
+    ((("h" "c"))("" ""))
     ((("t" "c"))("Ǯ"))
     ((("n" "c"))(""))
     ((("s" "c"))("Ƭ"))
     (((";" "c"))(""))
     ((("q" "c"))(""))
-    ((("j" "c"))(""))
+    ((("j" "c"))("" ""))
     ((("k" "c"))(""))
     ((("x" "c"))(""))
     ((("b" "c"))(""))
@@ -738,13 +738,13 @@
     ((("u" "r"))(""))
     ((("i" "r"))(""))
     ((("d" "r"))(""))
-    ((("h" "r"))(""))
+    ((("h" "r"))("" ""))
     ((("t" "r"))(""))
     ((("n" "r"))(""))
     ((("s" "r"))(""))
     (((";" "r"))(""))
     ((("q" "r"))(""))
-    ((("j" "r"))(""))
+    ((("j" "r"))("" ""))
     ((("k" "r"))(""))
     ((("x" "r"))(""))
     ((("b" "r"))(""))
@@ -766,7 +766,7 @@
     ((("," "l"))(""))
     ((("." "l"))(""))
     ((("p" "l"))(""))
-    ((("y" "l"))(""))
+    ((("y" "l"))("" ""))
     ((("f" "l"))(""))
     ((("g" "l"))(""))
     ((("c" "l"))(""))
@@ -793,7 +793,7 @@
     ((("v" "l"))(""))
     ((("z" "l"))(""))
     ((("/" "l"))(""))
-    ((("1" "a"))(""))
+    ((("1" "a"))("" ""))
     ((("2" "a"))(""))
     ((("3" "a"))(""))
     ((("4" "a"))(""))
@@ -811,20 +811,20 @@
     ((("g" "a"))(""))
     ((("c" "a"))(""))
     ((("r" "a"))(""))
-    ((("l" "a"))(""))
+    ((("l" "a"))("" ""))
     ((("a" "a"))(""))
-    ((("o" "a"))(""))
+    ((("o" "a"))("" ""))
     ((("e" "a"))(""))
     ((("u" "a"))(""))
-    ((("i" "a"))(""))
+    ((("i" "a"))("" ""))
     ((("d" "a"))(""))
     ((("h" "a"))("3"))
     ((("t" "a"))(""))
     ((("n" "a"))(""))
     ((("s" "a"))(""))
-    (((";" "a"))(""))
+    (((";" "a"))("" ""))
     ((("q" "a"))(""))
-    ((("j" "a"))(""))
+    ((("j" "a"))("" ""))
     ((("k" "a"))(""))
     ((("x" "a"))(""))
     ((("b" "a"))(""))
@@ -832,7 +832,7 @@
     ((("w" "a"))(""))
     ((("v" "a"))(""))
     ((("z" "a"))(""))
-    ((("/" "a"))(""))
+    ((("/" "a"))("" ""))
     ((("1" "o"))(""))
     ((("2" "o"))("Ǽ"))
     ((("3" "o"))(""))
@@ -847,7 +847,7 @@
     ((("." "o"))(""))
     ((("p" "o"))(""))
     ((("y" "o"))(""))
-    ((("f" "o"))(""))
+    ((("f" "o"))("" ""))
     ((("g" "o"))(""))
     ((("c" "o"))(""))
     ((("r" "o"))(""))
@@ -864,7 +864,7 @@
     ((("s" "o"))(""))
     (((";" "o"))(""))
     ((("q" "o"))(""))
-    ((("j" "o"))(""))
+    ((("j" "o"))("" ""))
     ((("k" "o"))(""))
     ((("x" "o"))(""))
     ((("b" "o"))(""))
@@ -886,7 +886,7 @@
     ((("," "e"))("Ȭ"))
     ((("." "e"))(""))
     ((("p" "e"))(""))
-    ((("y" "e"))(""))
+    ((("y" "e"))("" ""))
     ((("f" "e"))(""))
     ((("g" "e"))(""))
     ((("c" "e"))(""))
@@ -902,9 +902,9 @@
     ((("t" "e"))("!"))
     ((("n" "e"))(""))
     ((("s" "e"))(""))
-    (((";" "e"))(""))
+    (((";" "e"))("" ""))
     ((("q" "e"))(""))
-    ((("j" "e"))(""))
+    ((("j" "e"))("" ""))
     ((("k" "e"))("("))
     ((("x" "e"))(""))
     ((("b" "e"))(""))
@@ -924,7 +924,7 @@
     ((("9" "u"))(""))
     ((("0" "u"))(""))
     ((("," "u"))(""))
-    ((("." "u"))(""))
+    ((("." "u"))("" ""))
     ((("p" "u"))(""))
     ((("y" "u"))(""))
     ((("f" "u"))(""))
@@ -993,7 +993,7 @@
     ((("v" "i"))(""))
     ((("z" "i"))(""))
     ((("/" "i"))("Ư"))
-    ((("1" "d"))(""))
+    ((("1" "d"))("" ""))
     ((("2" "d"))(""))
     ((("3" "d"))(""))
     ((("4" "d"))(""))
@@ -1004,10 +1004,10 @@
     ((("9" "d"))("ʩ"))
     ((("0" "d"))(""))
     ((("," "d"))(""))
-    ((("." "d"))(""))
+    ((("." "d"))("" ""))
     ((("p" "d"))(""))
-    ((("y" "d"))(""))
-    ((("f" "d"))(""))
+    ((("y" "d"))("" ""))
+    ((("f" "d"))("" ""))
     ((("g" "d"))(""))
     ((("c" "d"))(""))
     ((("r" "d"))("³"))
@@ -1018,17 +1018,17 @@
     ((("u" "d"))("4"))
     ((("i" "d"))(")"))
     ((("d" "d"))(""))
-    ((("h" "d"))(""))
+    ((("h" "d"))("" ""))
     ((("t" "d"))(""))
-    ((("n" "d"))(""))
+    ((("n" "d"))("" ""))
     ((("s" "d"))(""))
     (((";" "d"))("1"))
     ((("q" "d"))(""))
     ((("j" "d"))(""))
-    ((("k" "d"))(""))
+    ((("k" "d"))("" ""))
     ((("x" "d"))(""))
     ((("b" "d"))(""))
-    ((("m" "d"))(""))
+    ((("m" "d"))("" ""))
     ((("w" "d"))(""))
     ((("v" "d"))("Ⱦ"))
     ((("z" "d"))(""))
@@ -1046,7 +1046,7 @@
     ((("," "h"))(""))
     ((("." "h"))("ʿ"))
     ((("p" "h"))("ʪ"))
-    ((("y" "h"))(""))
+    ((("y" "h"))("" ""))
     ((("f" "h"))(""))
     ((("g" "h"))(""))
     ((("c" "h"))(""))
@@ -1091,20 +1091,20 @@
     ((("g" "t"))(""))
     ((("c" "t"))(""))
     ((("r" "t"))(""))
-    ((("l" "t"))(""))
+    ((("l" "t"))("" ""))
     ((("a" "t"))(""))
     ((("o" "t"))(""))
     ((("e" "t"))(""))
     ((("u" "t"))(""))
     ((("i" "t"))(""))
-    ((("d" "t"))(""))
+    ((("d" "t"))("" ""))
     ((("h" "t"))(""))
-    ((("t" "t"))(""))
+    ((("t" "t"))("" ""))
     ((("n" "t"))(""))
     ((("s" "t"))(""))
     (((";" "t"))("ǯ"))
     ((("q" "t"))(""))
-    ((("j" "t"))(""))
+    ((("j" "t"))("" ""))
     ((("k" "t"))("8"))
     ((("x" "t"))(""))
     ((("b" "t"))(""))
@@ -1153,7 +1153,7 @@
     ((("v" "n"))(""))
     ((("z" "n"))(""))
     ((("/" "n"))(""))
-    ((("1" "s"))(""))
+    ((("1" "s"))("" ""))
     ((("2" "s"))(""))
     ((("3" "s"))(""))
     ((("4" "s"))(""))
@@ -1165,12 +1165,12 @@
     ((("0" "s"))(""))
     ((("," "s"))(""))
     ((("." "s"))(""))
-    ((("p" "s"))(""))
-    ((("y" "s"))(""))
+    ((("p" "s"))("" ""))
+    ((("y" "s"))("" ""))
     ((("f" "s"))(""))
     ((("g" "s"))(""))
     ((("c" "s"))("¢"))
-    ((("r" "s"))(""))
+    ((("r" "s"))("" ""))
     ((("l" "s"))(""))
     ((("a" "s"))(""))
     ((("o" "s"))(""))
@@ -1182,10 +1182,10 @@
     ((("t" "s"))(""))
     ((("n" "s"))(""))
     ((("s" "s"))(""))
-    (((";" "s"))(""))
+    (((";" "s"))("" ""))
     ((("q" "s"))(""))
-    ((("j" "s"))(""))
-    ((("k" "s"))(""))
+    ((("j" "s"))("" ""))
+    ((("k" "s"))("" ""))
     ((("x" "s"))(""))
     ((("b" "s"))("¤"))
     ((("m" "s"))(""))
@@ -1222,7 +1222,7 @@
     ((("t" ";"))(""))
     ((("n" ";"))(""))
     ((("s" ";"))(""))
-    (((";" ";"))(""))
+    (((";" ";"))("" ""))
     ((("q" ";"))(""))
     ((("j" ";"))(""))
     ((("k" ";"))(""))
@@ -1265,7 +1265,7 @@
     (((";" "q"))(""))
     ((("q" "q"))(""))
     ((("j" "q"))(""))
-    ((("k" "q"))(""))
+    ((("k" "q"))("" ""))
     ((("x" "q"))(""))
     ((("b" "q"))(""))
     ((("m" "q"))(""))
@@ -1334,7 +1334,7 @@
     ((("l" "k"))("¾"))
     ((("a" "k"))(""))
     ((("o" "k"))(""))
-    ((("e" "k"))(""))
+    ((("e" "k"))("" ""))
     ((("u" "k"))(""))
     ((("i" "k"))(""))
     ((("d" "k"))("̣"))
@@ -1352,8 +1352,8 @@
     ((("w" "k"))(""))
     ((("v" "k"))(""))
     ((("z" "k"))(""))
-    ((("/" "k"))(""))
-    ((("1" "x"))(""))
+    ((("/" "k"))("" ""))
+    ((("1" "x"))("" ""))
     ;((("2" "x"))(""))
     ;((("3" "x"))(""))
     ;((("4" "x"))(""))
@@ -1375,7 +1375,7 @@
     ((("a" "x"))(""))
     ((("o" "x"))(""))
     ((("e" "x"))("ˬ"))
-    ((("u" "x"))(""))
+    ((("u" "x"))("" ""))
     ((("i" "x"))(""))
     ((("d" "x"))(""))
     ((("h" "x"))(""))
@@ -1393,7 +1393,7 @@
     ((("v" "x"))(""))
     ((("z" "x"))("Ƚ"))
     ((("/" "x"))(""))
-    ((("1" "b"))(""))
+    ((("1" "b"))("" ""))
     ;((("2" "b"))(""))
     ;((("3" "b"))(""))
     ;((("4" "b"))(""))
@@ -1421,8 +1421,8 @@
     ((("h" "b"))(""))
     ((("t" "b"))(""))
     ((("n" "b"))(""))
-    ((("s" "b"))(""))
-    (((";" "b"))(""))
+    ((("s" "b"))("" ""))
+    (((";" "b"))("" ""))
     ((("q" "b"))("²"))
     ((("j" "b"))(""))
     ((("k" "b"))(""))
@@ -1449,7 +1449,7 @@
     ((("y" "m"))(""))
     ((("f" "m"))(""))
     ((("g" "m"))(""))
-    ((("c" "m"))(""))
+    ((("c" "m"))("" ""))
     ((("r" "m"))("ã"))
     ((("l" "m"))(""))
     ((("a" "m"))(""))
@@ -1483,7 +1483,7 @@
     ((("8" "w"))(""))
     ((("9" "w"))(""))
     ((("0" "w"))(""))
-    ((("," "w"))(""))
+    ((("," "w"))("" ""))
     ((("." "w"))(""))
     ((("p" "w"))(""))
     ((("y" "w"))(""))
@@ -1495,7 +1495,7 @@
     ((("a" "w"))(""))
     ((("o" "w"))("ɴ"))
     ((("e" "w"))(""))
-    ((("u" "w"))(""))
+    ((("u" "w"))("" ""))
     ((("i" "w"))(""))
     ((("d" "w"))(""))
     ((("h" "w"))("ʬ"))
@@ -1504,7 +1504,7 @@
     ((("s" "w"))(""))
     (((";" "w"))(""))
     ((("q" "w"))(""))
-    ((("j" "w"))(""))
+    ((("j" "w"))("" ""))
     ((("k" "w"))(""))
     ((("x" "w"))(""))
     ((("b" "w"))(""))
@@ -1513,14 +1513,14 @@
     ((("v" "w"))(""))
     ((("z" "w"))(""))
     ((("/" "w"))(""))
-    ((("1" "v"))(""))
+    ((("1" "v"))("" ""))
     ;((("2" "v"))(""))
     ;((("3" "v"))(""))
     ;((("4" "v"))(""))
     ;((("5" "v"))(""))
     ((("6" "v"))(""))
     ((("7" "v"))(""))
-    ((("8" "v"))(""))
+    ((("8" "v"))("" ""))
     ((("9" "v"))(""))
     ((("0" "v"))("Υ"))
     ((("," "v"))(""))
@@ -1553,7 +1553,7 @@
     ((("v" "v"))(""))
     ((("z" "v"))(""))
     ((("/" "v"))(""))
-    ((("1" "z"))(""))
+    ((("1" "z"))("" ""))
     ;((("2" "z"))(""))
     ((("3" "z"))(""))
     ;((("4" "z"))(""))
@@ -1563,9 +1563,9 @@
     ((("8" "z"))(""))
     ((("9" "z"))(""))
     ((("0" "z"))(""))
-    ((("," "z"))(""))
+    ((("," "z"))("" ""))
     ((("." "z"))(""))
-    ((("p" "z"))(""))
+    ((("p" "z"))("" ""))
     ((("y" "z"))("Ź"))
     ((("f" "z"))(""))
     ((("g" "z"))(""))
@@ -1592,7 +1592,7 @@
     ((("w" "z"))(""))
     ((("v" "z"))(""))
     ((("z" "z"))(""))
-    ((("/" "z"))(""))
+    ((("/" "z"))("" ""))
     ((("1" "/"))(""))
     ((("2" "/"))(""))
     ((("3" "/"))(""))
