@@ -1,6 +1,6 @@
 ;;; wnn.scm: Wnn for uim.
 ;;;
-;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2013 uim Project https://github.com/uim/uim
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -31,8 +31,7 @@
 
 (require "ustr.scm")
 (require "japanese.scm")
-(require "japanese-kana.scm")
-(require "japanese-azik.scm")
+(require "generic-predict.scm")
 (require-custom "generic-key-custom.scm")
 (require-custom "wnn-custom.scm")
 (require-custom "wnn-key-custom.scm")
@@ -43,18 +42,16 @@
 ;;
 ;; canna emulating functions
 ;;
-(define wnn-buf #f) ;; XXX
-
 (define (wnn-move wc-ctx nth)
   (wnn-lib-top wc-ctx)
   (if (< 0 nth)
       (for-each (lambda (x) (wnn-lib-move wc-ctx #t 'forward)) (iota nth))))
 (define (wnn-lib-init server)
   (if wnn-use-remote-server?
-      (set! wnn-buf (wnn-lib-open server "uim" wnn-rcfile 0))
-      (set! wnn-buf (wnn-lib-open "" "uim" wnn-rcfile 0))))
-(define (wnn-lib-alloc-context)
-  (wnn-lib-create-buffer wnn-buf 0 0))
+      (wnn-lib-open server "uim" wnn-rcfile 0)
+      (wnn-lib-open "" "uim" wnn-rcfile 0)))
+(define (wnn-lib-alloc-context wc)
+  (wnn-lib-create-buffer (wnn-context-wnn-buf wc) 0 0))
 (define (wnn-lib-get-nth-candidate wc seg nth)
   (let ((wc-ctx (wnn-context-wc-ctx wc)))
     (wnn-move wc-ctx seg)
@@ -62,7 +59,9 @@
     (wnn-lib-get-candidate wc-ctx nth)))
 (define (wnn-lib-release-context wc)
   (let ((wc-ctx (wnn-context-wc-ctx wc)))
-    (wnn-lib-destroy-buffer wc-ctx #f)))
+    (wnn-lib-destroy-buffer wc-ctx #t)
+    (wnn-lib-close (wnn-context-wnn-buf wc))
+    (wnn-context-set-wc-ctx! wc #f)))
 (define (wnn-lib-get-unconv-candidate wc seg-idx)
   (let ((wc-ctx (wnn-context-wc-ctx wc)))
     (wnn-move wc-ctx seg-idx)
@@ -90,8 +89,14 @@
     (wnn-lib-convert wc-ctx #f #f #t)
     (wnn-lib-get-nr-segments wc)))
 (define (wnn-lib-commit-segment wc seg delta)
-  (let ((wc-ctx (wnn-context-wc-ctx wc)))
+  (let* ((wc-ctx (wnn-context-wc-ctx wc)))
+    (predict-meta-commit
+     (wnn-context-prediction-ctx wc)
+     (cdr (assoc 'kanap (cdr (assoc 'clause-info (wnn-lib-get-jconvbuf wc-ctx)))))
+     (wnn-lib-get-candidate wc-ctx delta)
+     "")
     (wnn-lib-fix wc-ctx)
+    (wnn-lib-save-dic wc-ctx)
     (wnn-lib-clear wc-ctx)
     #t))
 (define (wnn-lib-reset-conversion wc)
@@ -99,8 +104,6 @@
     (wnn-lib-clear wc-ctx)))
 
 
-(define wnn-init-lib-ok? #f)
-
 (define wnn-type-direct	   ja-type-direct)
 (define wnn-type-hiragana	   ja-type-hiragana)
 (define wnn-type-katakana	   ja-type-katakana)
@@ -111,6 +114,8 @@
 (define wnn-input-rule-roma 0)
 (define wnn-input-rule-kana 1)
 (define wnn-input-rule-azik 2)
+(define wnn-input-rule-act 3)
+(define wnn-input-rule-kzik 4)
 
 (define wnn-candidate-type-katakana -2)
 (define wnn-candidate-type-hiragana -3)
@@ -285,6 +290,7 @@
 		      wnn-input-rule-kana))
 		 (lambda (wc)
 		   (wnn-prepare-input-rule-activation wc)
+                   (require "japanese-kana.scm")
 		   (wnn-context-set-input-rule! wc wnn-input-rule-kana)
                    (wnn-context-change-kana-mode!
                      wc (wnn-context-kana-mode wc))
@@ -301,10 +307,43 @@
 		      wnn-input-rule-azik))
 		 (lambda (wc)
 		   (wnn-prepare-input-rule-activation wc)
+                   (require "japanese-azik.scm")
 		   (rk-context-set-rule! (wnn-context-rkc wc)
 					 ja-azik-rule)
 		   (wnn-context-set-input-rule! wc wnn-input-rule-azik)))
 
+(register-action 'action_wnn_kzik
+		 (lambda (wc)
+		   '(ja_kzik
+		     ""
+		     "KZIK"
+		     "KZIKĥ޻ϥ⡼"))
+		 (lambda (wc)
+		   (= (wnn-context-input-rule wc)
+		      wnn-input-rule-kzik))
+		 (lambda (wc)
+		   (wnn-prepare-input-rule-activation wc)
+                   (require "japanese-kzik.scm")
+		   (rk-context-set-rule! (wnn-context-rkc wc)
+					 ja-kzik-rule)
+		   (wnn-context-set-input-rule! wc wnn-input-rule-kzik)))
+
+(register-action 'action_wnn_act
+		 (lambda (wc)
+		   '(ja_act
+		     ""
+		     "ACT"
+		     "ACTĥ޻ϥ⡼"))
+		 (lambda (wc)
+		   (= (wnn-context-input-rule wc)
+		      wnn-input-rule-act))
+		 (lambda (wc)
+		   (wnn-prepare-input-rule-activation wc)
+                   (require "japanese-act.scm")
+		   (rk-context-set-rule! (wnn-context-rkc wc)
+					 ja-act-rule)
+		   (wnn-context-set-input-rule! wc wnn-input-rule-act)))
+
 ;; Update widget definitions based on action configurations. The
 ;; procedure is needed for on-the-fly reconfiguration involving the
 ;; custom API
@@ -327,6 +366,8 @@
     (list 'state              #f)
     (list 'transposing        #f)
     (list 'transposing-type    0)
+    (list 'predicting         #f)
+    (list 'wnn-buf            #f)
     (list 'wc-ctx             #f) ;; wnn-internal-context
     (list 'preconv-ustr	      #f) ;; preedit strings
     (list 'rkc                ())
@@ -338,24 +379,68 @@
     (list 'alnum-type	      wnn-type-halfwidth-alnum)
     (list 'commit-raw         #t)
     (list 'input-rule         wnn-input-rule-roma)
-    (list 'raw-ustr	      #f))))
+    (list 'raw-ustr	      #f)
+    (list 'prediction-ctx     '())
+    (list 'prediction-word    '())
+    (list 'prediction-candidates '())
+    (list 'prediction-appendix '())
+    (list 'prediction-nr      '())
+    (list 'prediction-window  #f)
+    (list 'prediction-index   #f)
+    (list 'prediction-cache   '()))))
+
 (define-record 'wnn-context wnn-context-rec-spec)
 (define wnn-context-new-internal wnn-context-new)
 
+(define (wnn-predict wc str)
+  (predict-meta-search
+   (wnn-context-prediction-ctx wc)
+   str))
+(define (wnn-lib-set-prediction-src-string wc str)
+  (let* ((ret      (wnn-predict wc str))
+         (word     (predict-meta-word? ret))
+         (cands    (predict-meta-candidates? ret))
+         (appendix (predict-meta-appendix? ret)))
+    (wnn-context-set-prediction-word! wc word)
+    (wnn-context-set-prediction-candidates! wc cands)
+    (wnn-context-set-prediction-appendix! wc appendix)
+    (wnn-context-set-prediction-nr! wc (length cands)))
+  #f)
+(define (wnn-lib-get-nr-predictions wc)
+  (wnn-context-prediction-nr wc))
+(define (wnn-lib-get-nth-word wc nth)
+  (let ((word (wnn-context-prediction-word wc)))
+    (list-ref word nth)))
+(define (wnn-lib-get-nth-prediction wc nth)
+  (let ((cands (wnn-context-prediction-candidates wc)))
+    (list-ref cands nth)))
+(define (wnn-lib-get-nth-appendix wc nth)
+  (let ((appendix (wnn-context-prediction-appendix wc)))
+    (list-ref appendix nth)))
+(define (wnn-lib-commit-nth-prediction wc nth)
+  (predict-meta-commit
+   (wnn-context-prediction-ctx wc)
+   (wnn-lib-get-nth-word wc nth)
+   (wnn-lib-get-nth-prediction wc nth)
+   (wnn-lib-get-nth-appendix wc nth)))
+
+
 (define (wnn-context-new id im)
   (let ((wc (wnn-context-new-internal id im))
 	(rkc (rk-context-new ja-rk-rule #t #f)))
-;    (wnn-context-set-wc-ctx! wc (if wnn-init-lib-ok?
-;				      (wnn-lib-alloc-context) ()))
-    (wnn-context-set-wc-ctx! wc (wnn-lib-alloc-context))
+    (if (not (wnn-context-wnn-buf wc))
+        (wnn-context-set-wnn-buf! wc (wnn-lib-init wnn-server-name)))
+    (wnn-context-set-wc-ctx! wc (wnn-lib-alloc-context wc))
     (wnn-context-set-widgets! wc wnn-widgets)
     (wnn-context-set-rkc! wc rkc)
     (wnn-context-set-preconv-ustr! wc (ustr-new '()))
     (wnn-context-set-raw-ustr! wc (ustr-new '()))
     (wnn-context-set-segments! wc (ustr-new '()))
-    (if using-kana-table?
-        (wnn-context-set-input-rule! wc wnn-input-rule-kana)
-        (wnn-context-set-input-rule! wc wnn-input-rule-roma))
+    (if wnn-use-prediction?
+        (begin
+          (wnn-context-set-prediction-ctx! wc (predict-make-meta-search))
+          (predict-meta-open (wnn-context-prediction-ctx wc) "wnn")
+          (predict-meta-set-external-charset! (wnn-context-prediction-ctx wc) "EUC-JP")))
     wc))
 
 (define (wnn-commit-raw wc)
@@ -404,8 +489,10 @@
 	      (string-append-map-ustr-former extract-kana preconv-str)
 	      (if convert-pending-into-kana?
 		  (if residual-kana
-		      (extract-kana residual-kana)
-                      pending)
+                    (if (list? (car residual-kana))
+                      (string-append-map extract-kana residual-kana)
+		      (extract-kana residual-kana))
+                    pending)
 		  pending)
               (string-append-map-ustr-latter extract-kana preconv-str))))
 	   kana)
@@ -413,8 +500,10 @@
 	   (string-append-map-ustr-former extract-kana preconv-str)
            (if convert-pending-into-kana?
                (if residual-kana
-                   (extract-kana residual-kana)
-                   "")
+                 (if (list? (car residual-kana))
+                   (string-append-map extract-kana residual-kana)
+                   (extract-kana residual-kana))
+                 "")
                pending)
            (string-append-map-ustr-latter extract-kana preconv-str))))))
 
@@ -447,10 +536,6 @@
     (wnn-make-raw-string (wnn-get-raw-str-seq wc) wide? upper?)))
 
 (define (wnn-init-handler id im arg)
-  (if (not wnn-init-lib-ok?)
-      (begin
-	(wnn-lib-init wnn-server-name)
-	(set! wnn-init-lib-ok? #t)))
   (wnn-context-new id im))
 
 (define (wnn-release-handler wc)
@@ -465,9 +550,11 @@
   (ustr-clear! (wnn-context-segments wc))
   (wnn-context-set-transposing! wc #f)
   (wnn-context-set-state! wc #f)
-  (if (wnn-context-candidate-window wc)
+  (if (or (wnn-context-candidate-window wc)
+          (wnn-context-prediction-window wc))
       (im-deactivate-candidate-selector wc))
   (wnn-context-set-candidate-window! wc #f)
+  (wnn-context-set-prediction-window! wc #f)
   (wnn-context-set-candidate-op-count! wc 0))
 
 (define (wnn-begin-input wc key key-state)
@@ -522,8 +609,10 @@
 			      (wnn-context-transposing-state-preedit wc)
 			      (if (wnn-context-state wc)
 				  (wnn-compose-state-preedit wc)
-				  (wnn-input-state-preedit wc)))
-			  ())))
+                                  (if (wnn-context-predicting wc)
+                                      (wnn-predicting-state-preedit wc)
+                                      (wnn-input-state-preedit wc))))
+                          ())))
 	(context-update-preedit wc segments))
       (wnn-context-set-commit-raw! wc #f)))
 
@@ -663,7 +752,9 @@
 	         (res (rk-push-key! rkc key-str)))
 	    (if res
 	        (begin
-	          (ustr-insert-elem! (wnn-context-preconv-ustr wc) res)
+                  (if (list? (car res))
+                    (ustr-insert-seq! (wnn-context-preconv-ustr wc) res)
+                    (ustr-insert-elem! (wnn-context-preconv-ustr wc) res))
 	          (ustr-insert-elem! (wnn-context-raw-ustr wc) key-str))
 	        (if (null? (rk-context-seq rkc))
 		    (wnn-commit-raw wc)))))))))
@@ -776,6 +867,176 @@
 	   (wnn-flush wc)
 	   (wnn-proc-input-state wc key key-state))))))))
 
+(define (wnn-move-prediction wc offset)
+  (let* ((nr (wnn-lib-get-nr-predictions wc))
+         (idx (wnn-context-prediction-index wc))
+         (n (if (not idx)
+                0
+                (+ idx offset)))
+         (compensated-n (cond
+                         ((>= n nr)
+                          0)
+                        ((< n 0)
+                         (- nr 1))
+                        (else
+                         n))))
+    (im-select-candidate wc compensated-n)
+    (wnn-context-set-prediction-index! wc compensated-n)))
+
+(define (wnn-move-prediction-in-page wc numeralc)
+  (let* ((nr (wnn-lib-get-nr-predictions wc))
+         (p-idx (wnn-context-prediction-index wc))
+         (n (if (not p-idx)
+                0
+                p-idx))
+         (cur-page (if (= wnn-nr-candidate-max 0)
+                       0
+                       (quotient n wnn-nr-candidate-max)))
+         (pageidx (- (numeric-ichar->integer numeralc) 1))
+         (compensated-pageidx (cond
+                               ((< pageidx 0) ; pressing key_0
+                                (+ pageidx 10))
+                               (else
+                                pageidx)))
+         (idx (+ (* cur-page wnn-nr-candidate-max) compensated-pageidx))
+         (compensated-idx (cond
+                           ((>= idx nr)
+                            #f)
+                           (else
+                            idx)))
+         (selected-pageidx (if (not p-idx)
+                               #f
+                               (if (= wnn-nr-candidate-max 0)
+                                   p-idx
+                                   (remainder p-idx
+                                              wnn-nr-candidate-max)))))
+    (if (and
+         compensated-idx
+         (not (eqv? compensated-pageidx selected-pageidx)))
+        (begin
+          (wnn-context-set-prediction-index! wc compensated-idx)
+          (im-select-candidate wc compensated-idx)
+          #t)
+       #f)))
+
+(define (wnn-prediction-select-non-existing-index? wc numeralc)
+  (let* ((nr (wnn-lib-get-nr-predictions wc))
+         (p-idx (wnn-context-prediction-index wc))
+         (cur-page (if (= wnn-nr-candidate-max 0)
+                       0
+                       (quotient p-idx wnn-nr-candidate-max)))
+         (pageidx (- (numeric-ichar->integer numeralc) 1))
+         (compensated-pageidx (cond
+                               ((< pageidx 0) ; pressing key_0
+                                (+ pageidx 10))
+                               (else
+                                pageidx)))
+         (idx (+ (* cur-page wnn-nr-candidate-max) compensated-pageidx)))
+    (if (>= idx nr)
+        #t
+        #f)))
+
+(define (wnn-prediction-keys-handled? wc key key-state)
+  (cond
+   ((wnn-next-prediction-key? key key-state)
+    (wnn-move-prediction wc 1)
+    #t)
+   ((wnn-prev-prediction-key? key key-state)
+    (wnn-move-prediction wc -1)
+    #t)
+   ((and
+     wnn-select-prediction-by-numeral-key?
+     (ichar-numeric? key))
+    (wnn-move-prediction-in-page wc key))
+   ((and
+     (wnn-context-prediction-index wc)
+     (wnn-prev-page-key? key key-state))
+    (im-shift-page-candidate wc #f)
+    #t)
+   ((and
+     (wnn-context-prediction-index wc)
+     (wnn-next-page-key? key key-state))
+    (im-shift-page-candidate wc #t)
+    #t)
+   (else
+    #f)))
+
+(define (wnn-proc-prediction-state wc key key-state)
+  (cond
+   ;; prediction index change
+   ((wnn-prediction-keys-handled? wc key key-state))
+
+   ;; cancel
+   ((wnn-cancel-key? key key-state)
+    (if (wnn-context-prediction-index wc)
+        (wnn-reset-prediction-window wc)
+        (begin
+          (wnn-reset-prediction-window wc)
+          (wnn-proc-input-state wc key key-state))))
+
+   ;; commit
+   ((and
+     (wnn-context-prediction-index wc)
+     (wnn-commit-key? key key-state))
+    (wnn-do-commit-prediction wc))
+   (else
+    (if (and
+         wnn-use-implicit-commit-prediction?
+         (wnn-context-prediction-index wc))
+        (cond
+         ((or
+           ;; check keys used in wnn-proc-input-state-with-preedit
+           (wnn-begin-conv-key? key key-state)
+           (wnn-backspace-key? key key-state)
+           (wnn-delete-key? key key-state)
+           (wnn-kill-key? key key-state)
+           (wnn-kill-backward-key? key key-state)
+           (and
+            (not (wnn-context-alnum wc))
+            (wnn-commit-as-opposite-kana-key? key key-state))
+           (wnn-transpose-as-hiragana-key? key key-state)
+           (wnn-transpose-as-katakana-key? key key-state)
+           (wnn-transpose-as-halfkana-key? key key-state)
+           (and
+            (not (= (wnn-context-input-rule wc) wnn-input-rule-kana))
+            (or
+             (wnn-transpose-as-halfwidth-alnum-key? key key-state)
+             (wnn-transpose-as-fullwidth-alnum-key? key key-state)))
+           (wnn-hiragana-key? key key-state)
+           (wnn-katakana-key? key key-state)
+           (wnn-halfkana-key? key key-state)
+           (wnn-halfwidth-alnum-key? key key-state)
+           (wnn-fullwidth-alnum-key? key key-state)
+           (and
+            (not (wnn-context-alnum wc))
+            (wnn-kana-toggle-key? key key-state))
+           (wnn-alkana-toggle-key? key key-state)
+           (wnn-go-left-key? key key-state)
+           (wnn-go-right-key? key key-state)
+           (wnn-beginning-of-preedit-key? key key-state)
+           (wnn-end-of-preedit-key? key key-state)
+           (and
+            (modifier-key-mask key-state)
+            (not (shift-key-mask key-state))))
+          ;; go back to unselected prediction
+          (wnn-reset-prediction-window wc)
+          (wnn-check-prediction wc #f))
+         ((and
+           (ichar-numeric? key)
+           wnn-select-prediction-by-numeral-key?
+           (not (wnn-prediction-select-non-existing-index? wc key)))
+          (wnn-context-set-predicting! wc #f)
+          (wnn-context-set-prediction-index! wc #f)
+          (wnn-proc-input-state wc key key-state))
+         (else
+          ;; implicit commit
+          (wnn-do-commit-prediction wc)
+          (wnn-proc-input-state wc key key-state)))
+        (begin
+          (wnn-context-set-predicting! wc #f)
+          (wnn-context-set-prediction-index! wc #f)
+          (wnn-proc-input-state wc key key-state))))))
+
 (define (wnn-proc-input-state-with-preedit wc key key-state)
   (let ((preconv-str (wnn-context-preconv-ustr wc))
 	(raw-str (wnn-context-raw-ustr wc))
@@ -785,8 +1046,13 @@
     (cond
      ;; begin conversion
      ((wnn-begin-conv-key? key key-state)
+      (wnn-reset-prediction-window wc)
       (wnn-begin-conv wc))
 
+     ;; prediction
+     ((wnn-next-prediction-key? key key-state)
+      (wnn-check-prediction wc #t))
+
      ;; backspace
      ((wnn-backspace-key? key key-state)
       (if (not (rk-backspace rkc))
@@ -836,6 +1102,7 @@
 	   (or
 	    (wnn-transpose-as-halfwidth-alnum-key? key key-state)
 	    (wnn-transpose-as-fullwidth-alnum-key? key key-state))))
+      (wnn-reset-prediction-window wc)
       (wnn-proc-transposing-state wc key key-state))
 
      ((wnn-hiragana-key? key key-state)
@@ -936,22 +1203,6 @@
       #f)
 
      (else
-      ;; handle "n1" sequence as "1"
-      (if (and (not (wnn-context-alnum wc))
-	       (not (ichar-alphabetic? key))
-	       (not (string-find
-		     (rk-expect rkc)
-		     (charcode->string
-		      (if (= rule wnn-input-rule-kana)
-			  key
-			  (ichar-downcase key))))))
-	  (let ((pend (rk-pending rkc))
-		(residual-kana (rk-push-key-last! rkc)))
-	    (if residual-kana
-		(begin
-		  (ustr-insert-elem! preconv-str residual-kana)
-		  (ustr-insert-elem! raw-str pend)))))
-
       (if (wnn-context-alnum wc)
           (let ((key-str (charcode->string key))
 	        (pend (rk-pending rkc))
@@ -959,8 +1210,14 @@
 	    (rk-flush rkc) ;; OK to reset rkc here.
 	    (if residual-kana
 	        (begin
-		  (ustr-insert-elem! preconv-str residual-kana)
-		  (ustr-insert-elem! raw-str pend)))
+                  (if (list? (car residual-kana))
+                    (begin
+                      (ustr-insert-seq! preconv-str residual-kana)
+                      (ustr-insert-elem! raw-str (reverse
+                                                   (string-to-list pend))))
+                    (begin
+                      (ustr-insert-elem! preconv-str residual-kana)
+                      (ustr-insert-elem! raw-str pend)))))
 	    (ustr-insert-elem! preconv-str
 			       (if (= (wnn-context-alnum-type wc)
 				      wnn-type-halfwidth-alnum)
@@ -983,10 +1240,20 @@
 		      (ustr-insert-elem! preconv-str res))
 	          (if (and next-pend
 		           (not (string=? next-pend "")))
-		      (ustr-insert-elem! raw-str pend)
+		      (ustr-insert-seq! raw-str
+                                        (reverse (string-to-list pend)))
 		      (if (list? (car res))
 		          (begin
-			    (ustr-insert-elem! raw-str pend)
+                            (if (member pend
+                                        (map car
+                                             ja-consonant-syllable-table))
+                              ;; treat consonant having more than one
+                              ;; charactear as one raw-str in this case
+                              (ustr-insert-elem! raw-str pend)
+                              (ustr-insert-elem! raw-str (reverse
+                                                           (string-to-list
+                                                             pend))))
+                            ;; assume key-str as a vowel
 			    (ustr-insert-elem! raw-str key-str))
 		          (ustr-insert-elem!
 		           raw-str
@@ -1001,13 +1268,59 @@
 	       (residual-kana (rk-peek-terminal-match rkc)))
 	  (if residual-kana
 	      (begin
-		(ustr-insert-elem! preconv-str residual-kana)
+                (if (list? (car residual-kana))
+                  (ustr-insert-seq! preconv-str residual-kana)
+                  (ustr-insert-elem! preconv-str residual-kana))
 		(rk-flush rkc)))))))
 
+(define (wnn-reset-prediction-window wc)
+  (if (wnn-context-prediction-window wc)
+      (im-deactivate-candidate-selector wc))
+  (wnn-context-set-predicting! wc #f)
+  (wnn-context-set-prediction-window! wc #f)
+  (wnn-context-set-prediction-index! wc #f))
+
+(define (wnn-check-prediction wc force-check?)
+  (if (and
+       (not (wnn-context-state wc))
+       (not (wnn-context-transposing wc))
+       (not (wnn-context-predicting wc)))
+      (let* ((use-pending-rk-for-prediction? #t)
+	     (preconv-str
+	      (wnn-make-whole-string
+	       wc
+	       (not use-pending-rk-for-prediction?)
+	       (wnn-context-kana-mode wc)))
+	     (preedit-len (+
+			   (ustr-length (wnn-context-preconv-ustr wc))
+			   (if (not use-pending-rk-for-prediction?)
+			       0
+			       (string-length (rk-pending
+					       (wnn-context-rkc
+						wc)))))))
+	(if (or
+	     (>= preedit-len wnn-prediction-start-char-count)
+	     force-check?)
+	    (begin
+	      (wnn-lib-set-prediction-src-string wc preconv-str)
+	      (let ((nr (wnn-lib-get-nr-predictions wc)))
+		(if (and
+		     nr
+		     (> nr 0))
+		    (begin
+		      (im-activate-candidate-selector
+		       wc nr wnn-nr-candidate-max)
+		      (wnn-context-set-prediction-window! wc #t)
+		      (wnn-context-set-predicting! wc #t))
+		    (wnn-reset-prediction-window wc))))
+	    (wnn-reset-prediction-window wc)))))
+
 (define (wnn-proc-input-state wc key key-state)
   (if (wnn-has-preedit? wc)
       (wnn-proc-input-state-with-preedit wc key key-state)
-      (wnn-proc-input-state-no-preedit wc key key-state)))
+      (wnn-proc-input-state-no-preedit wc key key-state))
+  (if wnn-use-prediction?
+      (wnn-check-prediction wc #f)))
 
 (define wnn-separator
   (lambda (wc)
@@ -1050,8 +1363,10 @@
 	   (left-str (ustr-former-seq raw-str)))
      (append left-str
 	     (if residual-kana
-		 (list pending)
-		 '())
+               (if (list? (car residual-kana))
+                 (reverse (string-to-list pending))
+		 (list pending))
+               '())
 	      right-str))))
 
 (define wnn-get-raw-candidate
@@ -1076,7 +1391,9 @@
 	    (if (member (car unconv) preconv)
 		(let ((start (list-seq-contained? preconv unconv))
 		      (len (length unconv)))
-		  (if start
+		  (if (and
+                        start
+                        (= (length raw-str) (length preconv))) ;; sanity check
 		      (wnn-make-raw-string
 		       (reverse (sublist-rel raw-str start len))
 		       (if (or
@@ -1094,6 +1411,14 @@
 		"???") ;; FIXME
 	    "????")))))) ;; shouldn't happen
 
+(define (wnn-predicting-state-preedit wc)
+  (if (or
+       (not wnn-use-implicit-commit-prediction?)
+       (not (wnn-context-prediction-index wc)))
+      (wnn-input-state-preedit wc)
+      (let ((cand (wnn-get-prediction-string wc)))
+        (list (cons (bitwise-ior preedit-reverse preedit-cursor) cand)))))
+
 (define (wnn-compose-state-preedit wc)
   (let* ((segments (wnn-context-segments wc))
 	 (cur-seg (ustr-cursor-pos segments))
@@ -1168,6 +1493,22 @@
     (wnn-reset-candidate-window wc)
     (wnn-flush wc))
 
+(define (wnn-get-prediction-string wc)
+  (wnn-lib-get-nth-prediction
+   wc
+   (wnn-context-prediction-index wc)))
+
+(define (wnn-learn-prediction-string wc)
+  (wnn-lib-commit-nth-prediction
+   wc
+   (wnn-context-prediction-index wc)))
+
+(define (wnn-do-commit-prediction wc)
+  (im-commit wc (wnn-get-prediction-string wc))
+  (wnn-learn-prediction-string wc)
+  (wnn-reset-prediction-window wc)
+  (wnn-flush wc))
+
 (define wnn-correct-segment-cursor
   (lambda (segments)
     (if (ustr-cursor-at-end? segments)
@@ -1386,7 +1727,9 @@
               (wnn-proc-transposing-state wc key key-state)
               (if (wnn-context-state wc)
                   (wnn-proc-compose-state wc key key-state)
-                  (wnn-proc-input-state wc key key-state)))
+                  (if (wnn-context-predicting wc)
+                      (wnn-proc-prediction-state wc key key-state)
+                      (wnn-proc-input-state wc key key-state))))
 	  (wnn-proc-raw-state wc key key-state)))
   (wnn-update-preedit wc))
 
@@ -1406,13 +1749,19 @@
 ;;;
 (define (wnn-get-candidate-handler wc idx ascel-enum-hint)
   (let* ((cur-seg (ustr-cursor-pos (wnn-context-segments wc)))
-	 (cand (wnn-lib-get-nth-candidate
-		wc cur-seg idx)))
+         (cand (if (wnn-context-state wc)
+                   (wnn-lib-get-nth-candidate wc cur-seg idx)
+                   (wnn-lib-get-nth-prediction wc idx))))
     (list cand (digit->string (+ idx 1)) "")))
 
 (define (wnn-set-candidate-index-handler wc idx)
-  (ustr-cursor-set-frontside! (wnn-context-segments wc) idx)
-  (wnn-update-preedit wc))
+  (cond
+   ((wnn-context-state wc)
+    (ustr-cursor-set-frontside! (wnn-context-segments wc) idx)
+    (wnn-update-preedit wc))
+   ((wnn-context-predicting wc)
+    (wnn-context-set-prediction-index! wc idx)
+    (wnn-update-preedit wc))))
 
 (define (wnn-proc-raw-state wc key key-state)
   (if (not (wnn-begin-input wc key key-state))
